// src/implements/highlight-with-in-textarea.class.ts
var HighlightWithInTextarea = class {
  constructor(targetElement, config) {
    this.isGenerated = false;
    this.prefixIdForNewElement = void 0;
    this.targetElement = new HTMLTextAreaElement();
    this.highlightsElement = new HTMLElement();
    this.backdropElement = new HTMLElement();
    this.containerElement = new HTMLElement();
    this.highlightConfig = void 0;
    this.userBrowser = void 0;
    this.prefixIdForNewElement = `highlight-${this.generateUUIDv4WithoutHyphens()}`;
    this.targetElement = targetElement;
    if (config && config.highlight) {
      this.highlightConfig = config;
      this.generate();
    } else {
      console.error("valid config object not provided");
      throw new Error("valid config object not provided");
    }
  }
  generateUUIDv4WithoutHyphens() {
    return "xxxxxxxxxxxx4xxxyxxxxxxxxxxxxxxx".replace(/[xy]/g, function(c) {
      const r = Math.random() * 16 | 0, v = c === "x" ? r : r & 3 | 8;
      return v.toString(16);
    });
  }
  getType(instance) {
    let type = typeof instance;
    if (!instance) {
      return "falsely";
    } else if (Array.isArray(instance)) {
      if (instance.length === 2 && typeof instance[0] === "number" && typeof instance[1] === "number") {
        return "range";
      } else {
        return "array";
      }
    } else if (type === "object") {
      if (instance instanceof RegExp) {
        return "regexp";
      } else if (instance.hasOwnProperty("highlight")) {
        return "custom";
      }
    } else if (type === "function" || type === "string") {
      return type;
    }
    return "other";
  }
  generate() {
    this?.targetElement?.classList.add(this.prefixIdForNewElement + "-input", this.prefixIdForNewElement + "-content");
    this?.targetElement?.addEventListener("input", this.handleInput.bind(this));
    this?.targetElement?.addEventListener("scroll", this.handleScroll.bind(this));
    this.highlightsElement = document.createElement("div");
    this.highlightsElement.className = this.prefixIdForNewElement + "-highlights " + this.prefixIdForNewElement + "-content";
    this.backdropElement = document.createElement("div");
    this.backdropElement.className = this.prefixIdForNewElement + "-backdrop";
    this.backdropElement.appendChild(this.highlightsElement);
    this.containerElement = document.createElement("div");
    this.containerElement.className = this.prefixIdForNewElement + "-container";
    this.containerElement.appendChild(this.backdropElement);
    this.containerElement.appendChild(this.targetElement);
    this.containerElement.addEventListener("scroll", this.blockContainerScroll.bind(this));
    this.userBrowser = this.detectBrowser();
    switch (this.userBrowser) {
      case "firefox":
        this.fixFirefox();
        break;
      case "ios":
        this.fixIOS();
        break;
    }
    this.isGenerated = true;
    this.handleInput();
    document.body.appendChild(this.containerElement);
  }
  detectBrowser() {
    let ua = window.navigator.userAgent.toLowerCase();
    if (ua.indexOf("firefox") !== -1) {
      return "firefox";
    } else if (ua.includes("msie") || ua.includes("trident/7") || ua.includes("edge")) {
      return "ie";
    } else if (ua.includes("ipad") || ua.includes("iphone") || ua.includes("ipod") && !ua.includes("windows phone")) {
      return "ios";
    } else {
      return "other";
    }
  }
  fixFirefox() {
    let padding = window.getComputedStyle(this.highlightsElement);
    let border = window.getComputedStyle(this.highlightsElement);
    if (this.highlightsElement) {
      this.highlightsElement.style.padding = "0";
      this.highlightsElement.style.borderWidth = "0";
    }
    if (this.backdropElement) {
      this.backdropElement.style.marginTop = `+=${padding.getPropertyValue("padding-top")}`;
      this.backdropElement.style.marginRight = `+=${padding.getPropertyValue("padding-right")}`;
      this.backdropElement.style.marginBottom = `+=${padding.getPropertyValue("padding-bottom")}`;
      this.backdropElement.style.marginLeft = `+=${padding.getPropertyValue("padding-left")}`;
      this.backdropElement.style.marginTop = `+=${border.getPropertyValue("border-top-width")}`;
      this.backdropElement.style.marginRight = `+=${border.getPropertyValue("border-right-width")}`;
      this.backdropElement.style.marginBottom = `+=${border.getPropertyValue("border-bottom-width")}`;
      this.backdropElement.style.marginLeft = `+=${border.getPropertyValue("border-left-width")}`;
    }
  }
  fixIOS() {
    this.highlightsElement.style.paddingLeft = "3px";
    this.highlightsElement.style.paddingRight = "3px";
  }
  handleInput() {
    let input = this?.targetElement?.value;
    let ranges = this.getRanges(input, this.highlightConfig);
    let unStaggeredRanges = this.removeStaggeredRanges(ranges);
    let boundaries = this.getBoundaries(unStaggeredRanges);
    this.renderMarks(boundaries);
  }
  getRanges(input, highlight) {
    let type = this.getType(highlight);
    switch (type) {
      case "array":
        return this.getArrayRanges(input, highlight);
      case "function":
        return this.getFunctionRanges(input, highlight);
      case "regexp":
        return this.getRegExpRanges(input, highlight);
      case "string":
        return this.getStringRanges(input, highlight);
      case "range":
        return this.getRangeRanges(input, highlight);
      case "custom":
        return this.getCustomRanges(input, highlight);
      default:
        if (!highlight) {
          return [];
        } else {
          console.error("unrecognized highlight type");
          throw new Error("unrecognized highlight type");
        }
    }
  }
  getArrayRanges(input, arr) {
    let ranges = arr.map((range) => this.getRanges(input, range));
    return [].concat(...ranges);
  }
  getFunctionRanges(input, func) {
    return this.getRanges(input, func(input));
  }
  getRegExpRanges(input, regex) {
    let ranges = [];
    let match;
    while ((match = regex.exec(input)) !== null) {
      ranges.push([match.index, match.index + match[0].length]);
      if (!regex.global) {
        break;
      }
    }
    return ranges;
  }
  getStringRanges(input, str) {
    let ranges = [];
    let inputLower = input.toLowerCase();
    let strLower = str.toLowerCase();
    let index = 0;
    while ((index = inputLower.indexOf(strLower, index)) !== -1) {
      ranges.push([index, index + strLower.length]);
      index += strLower.length;
    }
    return ranges;
  }
  getRangeRanges(_, range) {
    return [range];
  }
  getCustomRanges(input, custom) {
    let ranges = this.getRanges(input, custom.highlight);
    if (custom.className) {
      ranges.forEach((range) => {
        if (range.className) {
          range.className = `${custom.className} ${range.className}`;
        } else {
          range.className = custom.className;
        }
      });
    }
    return ranges;
  }
  removeStaggeredRanges(ranges) {
    let unStaggeredRanges = [];
    ranges.forEach((range) => {
      let isStaggered = unStaggeredRanges.some((unStaggeredRange) => {
        let isStartInside = range[0] > unStaggeredRange[0] && range[0] < unStaggeredRange[1];
        let isStopInside = range[1] > unStaggeredRange[0] && range[1] < unStaggeredRange[1];
        return isStartInside !== isStopInside;
      });
      if (!isStaggered) {
        unStaggeredRanges.push(range);
      }
    });
    return unStaggeredRanges;
  }
  getBoundaries(ranges) {
    let boundaries = [];
    ranges.forEach(function(range) {
      boundaries.push({
        type: "start",
        index: range[0],
        className: range.className
      });
      boundaries.push({
        type: "stop",
        index: range[1]
      });
    });
    this.sortBoundaries(boundaries);
    return boundaries;
  }
  sortBoundaries(boundaries) {
    boundaries.sort((a, b) => {
      if (a.index !== b.index) {
        return b.index - a.index;
      } else if (a.type === "stop" && b.type === "start") {
        return 1;
      } else if (a.type === "start" && b.type === "stop") {
        return -1;
      } else {
        return 0;
      }
    });
  }
  renderMarks(boundaries) {
    let input = this?.targetElement?.value;
    if (input) {
      boundaries.forEach((boundary, index) => {
        let markup = (() => {
          if (boundary.type === "start") {
            return "{{hwt-mark-start|" + index + "}}";
          } else {
            return "{{hwt-mark-stop}}";
          }
        })();
        input = (input ?? "").slice(0, boundary.index) + markup + (input ?? "").slice(boundary.index);
      });
      input = input.replace(/\n({{hwt-mark-stop}})?$/, "\n\n$1");
      input = input.replace(/</g, "&lt;").replace(/>/g, "&gt;");
      if (this.userBrowser === "ie") {
        input = input.replace(/ /g, " <wbr>");
      }
      input = input.replace(/{{hwt-mark-start\|(\d+)}}/g, function(match, submatch) {
        const className = boundaries[+submatch].className;
        if (className) {
          return '<mark class="' + className + '">';
        } else {
          return "<mark>";
        }
      });
      input = input.replace(/{{hwt-mark-stop}}/g, "</mark>");
      this.highlightsElement.innerHTML = input;
    }
  }
  handleScroll() {
    if (this.backdropElement && this.targetElement) {
      this.backdropElement.scrollTop = this.targetElement.scrollTop;
      let scrollLeft = this.targetElement.scrollLeft;
      this.backdropElement.style.transform = scrollLeft > 0 ? `translateX(${-scrollLeft}px)` : "";
    }
  }
  blockContainerScroll() {
    if (this.containerElement) {
      this.containerElement.scrollLeft = 0;
    }
  }
  destroy() {
    this.backdropElement?.remove();
    this?.targetElement?.parentElement?.removeChild(this?.targetElement);
  }
};
export {
  HighlightWithInTextarea
};
//# sourceMappingURL=index.js.map
